<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/ProductExtraInfo.php");
	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImportBase.php");
	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_manufacturer.php");
	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_category.php");
	
	/*
	 * ex. : 
	  <product>
        <reference>albala</reference>
        <name>albala</name>
        <manufacturer ref="16384">arche</manufacturer>
        <category type="rfs" ref="835584">
            <category type="family" ref="606208">
                <category type="ray" ref="98304">
                    <name>maroquineries</name>
                </category>
                <name>sac</name>
            </category>
            <name>sac</name>
        </category>
        <category type="c1" ref="3833856">albia</category>
        <category type="season" ref="32768">permanent</category>
        <category type="collection" ref="32770">h14</category>
        <category type="collection" ref="327680">H12</category>
        <category type="collection" ref="65538">h15</category>
        <vat>20.00</vat>
        <ecotax>0.00</ecotax>
        <meta>chaussures femme arche emboitant confortable hiver 2016</meta>
        <description>Un emboitant stretch bi-matière asymétrique qui s'enfile.</description>
        <articles>
            <article sku="20015107">
                <size>36</size>
                <wp>65.25</wp>
                <sp>150.00</sp>
                <photos>42336257,42352641,42369025</photos>
            </article>
            <article sku="20015114">
                <size>37</size>
                <wp>65.25</wp>
                <sp>150.00</sp>
                <photos>42336257,42352641,42369025</photos>
            </article>
	  </product>
	 */

	class PolarisLinkImport_product extends PolarisLinkImportBase 
	{
		public static $_ATTRIBUTE_CLASS;

		private $Manufacturers = null;
		private $CategoryImporter = null;
		private $OrderedAttributesGroup;
		private $OrderedFeaturesGroup;
		private $SizeGridAttributesGroup;
		private $AttributesCache = null;
		private $FeaturesCache = null;

		private function addAttribute($id_attr, $xmlTag)
		{
			if ($id_attr > 0 && !empty($xmlTag))
				$this->OrderedAttributesGroup[$id_attr] = $xmlTag;
		}

		private function addFeature($id_feature, $xmlTag)
		{
			if ($id_feature > 0 && !empty($xmlTag))
				$this->OrderedFeaturesGroup[$id_feature] = $xmlTag;
		}
		
		protected function OnInit()
		{
			parent::OnInit();
			
			self::$_ATTRIBUTE_CLASS = (version_compare(_PS_VERSION_, "8") >= 0) ? "ProductAttribute" : "Attribute";
			$this->Tags = $this->JobManager->getImporter("tag");
			$this->Manufacturers = $this->JobManager->getImporter("manufacturer");
			$this->CategoryImporter = $this->JobManager->getImporter("category");
			
			$this->SizeGridAttributesGroup = array();
			if ($this->Link->OPT_SIZE_GRID)
			{
				// Chargement des attributs...
				foreach (AttributeGroup::getAttributesGroups($this->Link->LangID) as $attrGr)
					$this->SizeGridAttributesGroup[mb_strtolower($attrGr["name"])] = $attrGr["id_attribute_group"];
			}

			$this->OrderedAttributesGroup = array();
			$this->addAttribute(intval($this->Link->AttrGroupForAttr0), "size");
			$this->addAttribute(intval($this->Link->AttrGroupForAttr1), "lvl1");
			$this->addAttribute(intval($this->Link->AttrGroupForAttr2), "lvl2");
			$this->addAttribute(intval($this->Link->AttrGroupForAttr3), "lvl3");
			$this->addAttribute(intval($this->Link->AttrGroupForAttr4), "lvl4");
			$this->addAttribute(intval($this->Link->AttrGroupForAttrG1), "g_lvl1");
			$this->addAttribute(intval($this->Link->AttrGroupForAttrG2), "g_lvl2");
			$this->addAttribute(intval($this->Link->AttrGroupForAttrG3), "g_lvl3");
			$this->addAttribute(intval($this->Link->AttrGroupForAttrG4), "g_lvl4");
			ksort($this->OrderedAttributesGroup);

			$this->OrderedFeaturesGroup = array();
			$this->addFeature(intval($this->Link->FeatureForAttr1), "*lvl1");
			$this->addFeature(intval($this->Link->FeatureForAttr2), "*lvl2");
			$this->addFeature(intval($this->Link->FeatureForAttr3), "*lvl3");
			$this->addFeature(intval($this->Link->FeatureForAttr4), "*lvl4");
			$this->addFeature(intval($this->Link->FeatureForAttrG1), "*g_lvl1");
			$this->addFeature(intval($this->Link->FeatureForAttrG2), "*g_lvl2");
			$this->addFeature(intval($this->Link->FeatureForAttrG3), "*g_lvl3");
			$this->addFeature(intval($this->Link->FeatureForAttrG4), "*g_lvl4");

			$this->addFeature(intval($this->Link->FeatureForCategory1), "!c1");
			$this->addFeature(intval($this->Link->FeatureForCategory2), "!c2");
			$this->addFeature(intval($this->Link->FeatureForCategory3), "!c3");
			$this->addFeature(intval($this->Link->FeatureForCategory4), "!c4");
			$this->addFeature(intval($this->Link->FeatureForSeason), "!season");
			$this->addFeature(intval($this->Link->FeatureForCollection), "!collection");

			ksort($this->OrderedFeaturesGroup);
		}			

		protected function BuildImage($product, $polarisPhotoId)
		{
			$image = new Image();
			$image->id_product = (int)($product->id);
			$image->position = Image::getHighestPosition($product->id) + 1;
			$image->cover = false;

			if (!$image->add()) 
			{
				$this->logError(sprintf("impossible d'ajouter une photo au produit %s (%s)", $product->id, Db::getInstance()->getMsgError()));
				return NULL;
			} 
			else 
			{
				// Associate image to shop from context
				$image->associateTo(Shop::getContextListShopID());

				$imagePath = $image->getPathForCreation();
				if (!$imagePath) 
				{
					$this->logError(sprintf("impossible de créer le dossier de destination de la photo pour le produit %s", $product->id));
					$image->delete();
					return NULL;
				}

				$imagePath = dirname($imagePath) . DIRECTORY_SEPARATOR;

				if (!$this->createPSImg($imagePath . $image->id . "." . $image->image_format, $polarisPhotoId, $product))
				{
					$image->delete();
					return NULL;				
				}
			
				if (!$image->update()) 
				{
					$this->logError("erreur lors de la mise à jour du statut de l'image");
					$image->delete();
					return NULL;
				}				
				
				// Génération des thumbnails et nettoyage webp				
				$this->RebuildThumbnails("products", "product", $imagePath, $image->id_product, $image->id, $image->image_format);
					
				// Plugin, watermark ?
				Hook::exec('actionWatermark', array('id_image' => $image->id, 'id_product' => $product->id));

				return $image;
			}
		}
		
		public static function sortAttributesNatural($langId, $groupId)
		{
			if (empty(self::$_ATTRIBUTE_CLASS))
				self::$_ATTRIBUTE_CLASS = (version_compare(_PS_VERSION_, "8") >= 0) ? "ProductAttribute" : "Attribute";

			$sizesSorter = array("tu" => 1, "xxxxs" => 2, "xxxs" => 3, "xxs" => 4, "xs" => 5, "s" => 6, "m" => 7, "l" => 8, "xl" => 9, "xxl" => 10, "xxxl" => 11, "xxxxl" => 12);
			
			$attributes = array();
						
			foreach (AttributeGroup::getAttributes($langId, $groupId) as $attr)
			{
				$name = mb_strtolower(trim($attr["name"]));
				if (isset($sizesSorter[$name]))
					$name = $sizesSorter[$name];
				
				$attributes[$attr["id_attribute"]] = $name;
			}
						
			uasort($attributes, function($a, $b) {
									return strnatcmp($a, $b);
								});

			$position = 0;
			foreach ($attributes as $k => $v)
			{
				$attribute = new self::$_ATTRIBUTE_CLASS($k);
				$attribute->position = $position++;
				$attribute->save();
			}			
		}

		protected function getOrCreateAttribute($groupId, $names, $color = NULL, $position = NULL)
		{
			if ($this->AttributesCache == null)
			{
				// Chargement du cache d'attributs
				$id_lang = $this->Link->LangID;
				$attributesList = $this->SafeSQL('
										SELECT DISTINCT ag.*, agl.*, a.`id_attribute`, a.color, a.position, al.`name`, agl.`name` AS `attribute_group`
										FROM `'._DB_PREFIX_.'attribute_group` ag
										LEFT JOIN `'._DB_PREFIX_.'attribute_group_lang` agl
											ON (ag.`id_attribute_group` = agl.`id_attribute_group` AND agl.`id_lang` = '.(int)$id_lang.')
										LEFT JOIN `'._DB_PREFIX_.'attribute` a
											ON a.`id_attribute_group` = ag.`id_attribute_group`
										LEFT JOIN `'._DB_PREFIX_.'attribute_lang` al
											ON (a.`id_attribute` = al.`id_attribute` AND al.`id_lang` = '.(int)$id_lang.')
										'.Shop::addSqlAssociation('attribute_group', 'ag').'
										'.Shop::addSqlAssociation('attribute', 'a').'
										ORDER BY agl.`name` ASC, a.`position` ASC
									', true, false);
				
				foreach ($attributesList as $attribute)
				{
					if (!isset($this->AttributesCache[$attribute["id_attribute_group"]]))
						$this->AttributesCache[$attribute["id_attribute_group"]] = array();
					
					$this->AttributesCache[$attribute["id_attribute_group"]][strtolower(trim($attribute["name"]))] = $attribute;
				}
			}
			
			if (isset($this->AttributesCache[$groupId][strtolower(trim($names[$this->Link->LangID]))]))
			{
				// Il existe 
				$attr = $this->AttributesCache[$groupId][strtolower(trim($names[$this->Link->LangID]))];
				
				// On vérifie la couleur
				if (!empty($color) && $color != $attr["color"])
				{
					$attribute = new self::$_ATTRIBUTE_CLASS($attr["id_attribute"]);
					$attribute->color = $color;
					$attribute->save();
					// Mise à jour du cache
					$this->AttributesCache[$groupId][strtolower(trim($names[$this->Link->LangID]))]["color"] = $color;
				}				
				
				return $attr["id_attribute"];
			}
			
			// L'attribut n'existe pas ? On le crée !
			$attribute = new self::$_ATTRIBUTE_CLASS();
			$attribute->id_attribute_group = $groupId;
			$attribute->position = $position;
			$attribute->color = $color;
			$attribute->name = $this->AutoFillEmptyLangValue($names);
			$attribute->save();
			
			// Mise à jour du cache
			$this->AttributesCache[$groupId][strtolower(trim($names[$this->Link->LangID]))] = array(
																								"id_attribute" => $attribute->id,
																								"id_attribute_group" => $groupId,
																								"position" => $position,
																								"color" => $color
																							);
			return $attribute->id;
		}

		protected function getOrCreateFeature($featureId, $names)
		{
			if ($this->FeaturesCache == null)
			{
				// Chargement du cache d'attributs
				$id_lang = $this->Link->LangID;
				$featuresValueList = $this->SafeSQL('
										SELECT DISTINCT 
												fv.id_feature, fvl.id_feature_value, fvl.value
										FROM `'._DB_PREFIX_.'feature_value_lang` fvl
										LEFT JOIN `'._DB_PREFIX_.'feature_value` fv
											ON (fv.`id_feature_value` = fvl.`id_feature_value` AND fvl.`id_lang` = '.(int)$id_lang.')
									', true, false);
				
				foreach ($featuresValueList as $featureValue)
				{
					if (!isset($this->FeaturesCache[$featureValue["id_feature"]]))
						$this->FeaturesCache[$featureValue["id_feature"]] = array();
					
					$this->FeaturesCache[$featureValue["id_feature"]][strtolower(trim($featureValue["value"]))] = $featureValue;
				}
			}
			
			if (isset($this->FeaturesCache[$featureId][strtolower(trim($names[$this->Link->LangID]))]))
			{
				// Il existe 
				$attr = $this->FeaturesCache[$featureId][strtolower(trim($names[$this->Link->LangID]))];				
				return $attr["id_feature_value"];
			}
						
			// L'attribut n'existe pas ? On le crée !
			$featureValue = new FeatureValue();
			$featureValue->id_feature = $featureId;
			$featureValue->value = $this->AutoFillEmptyLangValue($names);
			$featureValue->save();
			
			// Mise à jour du cache
			$this->FeaturesCache[$featureId][strtolower(trim($names[$this->Link->LangID]))] = array(
																								"id_feature_value" => $featureValue->id,
																								"id_feature" => $featureId
																							);
			return $featureValue->id;
		}
		
		
		protected function associateProductToPolarisRef($polarisRef, $productId, $productAttributeId = 0)
		{
			$productSupplier = ProductSupplier::getIdByProductAndSupplier($productId, $productAttributeId, $this->Link->SupplierID);
			if ($productSupplier)
			{
				$productSupplier = new ProductSupplier($productSupplier);
				if ($productSupplier->product_supplier_reference != $polarisRef || $productSupplier->id_currency != $this->Link->CurrencyID)
				{
					$productSupplier->product_supplier_reference = $polarisRef;
					$productSupplier->id_currency = $this->Link->CurrencyID;
					$productSupplier->save();
				}
			}
			else
			{
				$productSupplier = new ProductSupplier();
				$productSupplier->id_supplier = $this->Link->SupplierID;
				$productSupplier->id_product = $productId;
				$productSupplier->id_product_attribute = $productAttributeId;
				$productSupplier->product_supplier_reference = $polarisRef;
				$productSupplier->id_currency = $this->Link->CurrencyID;
				$productSupplier->add();
			}
			
			// Entretien du cache
			if ($this->RefsCache != null)
				$this->RefsCache->setRef($polarisRef, $productId, $productAttributeId);
		}				
		
		public function reassociateAttributes($combination, $ids_attribute)
		{
			$result = Db::getInstance()->delete('product_attribute_combination', '`id_product_attribute` = '.(int)$combination->id);

			if ($result && !empty($ids_attribute)) {
				$sql_values = array();
				foreach ($ids_attribute as $value) {
					$sql_values[] = '('.(int)$value.', '.(int)$combination->id.')';
				}

				$result = Db::getInstance()->execute('
					INSERT INTO `'._DB_PREFIX_.'product_attribute_combination` (`id_attribute`, `id_product_attribute`)
					VALUES '.implode(',', $sql_values)
				);
			}
			return $result;
		}
		
		protected function OnImportData($data)
		{			
			if (!$this->Link->OPT_ALLOW_I_CATALOG)
				return;	// Rien à faire ici dans ce cas là !
			
			if ($data->articles == null || count($data->articles) == 0)
				return;	// Rien à importer !
			
			// On se positionne sur le bon shop !
			$this->SetCatalogContext();
			
			// Localisation des produits concernés ...
			$db = Db::getInstance();
			$referencesList = array();
			$eansList = array();
			$refPublished = 0;
			$minPA = 0;
			$minPV = 0;
			$photosList = array();
			
			foreach ($data->articles->article as $article)
			{
				$sku = (string) $article["sku"];
				if (!empty($sku))
				{
					$referencesList[] = trim($sku);
					if (!empty($sku) && (int) $article["deleted"] == 0)
					{
						$refPublished += 1;
						// On regarde les prix ?
						$pa = floatval((string) $article->wp);
						if ($minPA == 0 || ($pa > 0 && $minPA > $pa))
							$minPA = $pa;
						
						$pv = floatval((string) $article->sp);
						if ($minPV == 0 || ($pv > 0 && $minPV > $pv))
							$minPV = $pv;
						
						$ean = (string) $article["ean"];
						if (!empty($ean))
							$eansList[] = $ean;	

						if (!empty((string) $article->photos))
							$photosList = array_merge($photosList, $this->GetReferencesFromList((string) $article->photos));
					}
				}
			}

			if (empty($referencesList))
				return;	// Rien à faire

			$references = join(',', $referencesList);
			$eans = join(',', $eansList);
			
			// On ne localise QUE par EAN (et pas par GENCODE) pour le catalogue, sinon catastrophe !!
			// (on ne met à jour que les références que le pont a créé, le reste c'est pour reconnaitre pour les stocks et les prix)
			$isOnCreation = false;

			$product = $this->GetProduct($this->GetProductCombinationFor($references, NULL, false, false, true));			
			if ($this->RefsCache != null && !empty($product))
			{
				// Ménage possible quand refCache activé, on va supprimer tous les produits en double qui ne correspondent pas à celui trouvé
				foreach ($this->RefsCache->getAll($referencesList) as $p)
				{
					if ($p["id_product"] != $product->id)
					{						
						$p2delete = new Product($p["id_product"]);
						if (!empty($p2delete->id))
						{
							$p2delete->delete();
							$this->Log($product, "Fusion du produit %s dans le produit %s", [$p["id_product"], $product->id]);
						}
						$this->RefsCache->setRef($p["ref"], null, null);
					}
				}
			}
			
			if (!empty($refPublished))
			{
				if ($product == NULL)
				{					
					$product = new Product();
					$product->active = false;
					$isOnCreation = true;
					
					$product->reference = checkProductRef((string) $this->GetMandatory($data, "reference"));
				}

				$log = $this->NewLogCtx($product, "%s du produit %s", [ $isOnCreation ? "Création" : "Modification", $product->reference]);
								
				$polarisRef = $this->GetFirstReferencesFromList($references);
				$this->updProperty($log, $product, "id_supplier", $this->Link->SupplierID);
				$this->updProperty($log, $product, "supplier_reference", $polarisRef);
				$this->updProperty($log, $product, "reference", $this->GetMandatory($data, "reference"), 'checkProductRef');
				
				if (!empty($eans))
					$this->updProperty($log, $product, "ean13", checkEAN13($this->GetFirstReferencesFromList($eans)));
				
				if ($isOnCreation || $this->Link->OPT_ALLOW_I_NAME)
					$this->updProperty($log, $product, "name", $this->GetMandatory($data, "name"), "checkProductName", null, true);				

				// Type de produit
				$is_virtual = $this->GetOptional($data, "type") == "service";
				$this->updProperty($log, $product, "is_virtual", $is_virtual ? 1 : 0);
				
				// Disponible à la vente !
				$this->updProperty($log, $product, "available_for_order", "1");

				// Infos avancées du produit
				if ($this->Link->OPT_UNPUBLISH_COOLDOWN)
				{
					$extraInfos = ProductExtraInfo::GetForProduct($product->id);
					if ($extraInfos->date_depublication)
					{
						// Annulation de la dépublication !
						$extraInfos->date_depublication = NULL;
						$extraInfos->save();
					}
				}

				// Prix et taxes
				$id_tax_rule = 0;
				if (isset($data->vat["rule"]))
					$id_tax_rule = (string) $data->vat["rule"];

				if ((empty($id_tax_rule) || !$this->JobManager->isTaxesRulesExists($id_tax_rule)) && !empty((string) $data->vat))
				{
					// Pas trouvé ? On essaye de deviner la règle ...
					$id_country = NULL;
					if (!empty((string) $data->vat["country"]))
						$id_country = (int) $this->JobManager->getCountryForISO((string) $data->vat["country"]);
									
					// Pas de pays, on prend celui du magasin par défaut !
					if (empty($id_country))
						$id_country = (int) Configuration::get('PS_COUNTRY_DEFAULT');
					
					$id_tax_rule = $this->JobManager->getTaxesGroupFor($id_country, (string) $data->vat);
				}

				if (empty($id_tax_rule))
					throw new Exception(sprintf("impossible de localiser la taxe de %s pour paramétrer le produit %s", (string) $data->vat, $product->reference));
				else if (!$this->JobManager->isTaxesRulesExists($id_tax_rule))
					throw new Exception(sprintf("la règle de taxes n°%s n'existe pas !", (string) $id_tax_rule));
				
				$this->updProperty($log, $product, "id_tax_rules_group", $id_tax_rule);				
				
				$this->updProperty($log, $product, "price", checkPrice($minPV));
				$this->updProperty($log, $product, "wholesale_price", checkPrice($minPA));
				$this->updOptProperty($log, $product, "ecotax", $data, "ecotax");				

				// Poids par défaut
				if ($this->Link->DefaultWeigth > 0 && (!isset($data->weight) || $data->weight <= 0))
					$data->weight = round($this->Link->DefaultWeigth / 1000.0, 3);

				$this->updOptProperty($log, $product, "weight", $data, "weight");
				$this->updOptProperty($log, $product, "width", $data, "width");
				$this->updOptProperty($log, $product, "height", $data, "height");
				$this->updOptProperty($log, $product, "depth", $data, "depth");
				
				if ($isOnCreation || $this->Link->OPT_ALLOW_I_MDESCRIPTIONS)
					$this->updOptProperty($log, $product, "meta_description", $data, "meta", null, 'checkMetaDescription');
				if ($isOnCreation || $this->Link->OPT_ALLOW_I_DESCRIPTIONS)
					$this->updOptProperty($log, $product, "description", $data, "description", null, 'checkDescription');

				// Liaison marque
				if (!empty((string) $data->manufacturer["ref"]))
					$this->updProperty($log, $product, "id_manufacturer", $this->Manufacturers->getOrCreateManufacturerId((string) $data->manufacturer["ref"], (string) $data->manufacturer));
				
				if ($isOnCreation || $this->Link->OPT_ALLOW_I_PRODUCT_LINK)
				{
					$lnk_langs = [];

					if (empty($this->Link->SlugBuildMethod))
					{
						foreach ($product->name as $k => $v)
							$lnk_langs[$k] = $this->BuildLinkRewrite($data->reference.' '.$data->manufacturer->__toString());
					}
					else
					{
						foreach ($product->name as $k => $v)
						{
							$GLOBALS["__data-lang"] = $k;
							$GLOBALS["__data-v"] = $data;
							$lnk_langs[$k] = 
								trim(preg_replace('/-{2,}/','-',	// test---test => test-test
									$this->BuildLinkRewrite(preg_replace_callback('|\{([a-z0-9-_]+)\}|', function ($matches) {
										$data = $GLOBALS["__data-v"];
										// cas simple
										switch ($matches[1])
										{
											case "reference": return (string) $data->reference; break;
											case "name": return (string) $data->name; break;
											case "manufacturer": return (string) $data->manufacturer; break;
											
										}

										if (substr($matches[1], 0, 6) == "niveau")
										{
											$lvl = str_replace("niveau", "lvl", $matches[1]);
											$allTheSame = "";
											foreach ($data->articles->article as $article)
											{
												if (empty($article["deleted"]))
												{
													if ((string) $article->$lvl != $allTheSame && !empty($allTheSame))
														return "";	// Pas possible, niveaux différents !
													$allTheSame = (string) $article->$lvl;
												}
											}
											return $allTheSame;
										}																			

										// catégorie
										foreach ($data->category as $category)
										{
											if ((string) $category["type"] == $matches[1])
											{
												// C'est une catégorie !!
												return (string) $category->name;
											}
										}																			

										return "";
									}, mb_strtolower($this->Link->SlugBuildMethod)))), '-');
						}						
					}					

					// Dédoublonnage !
					foreach ($lnk_langs as $k => $v)
					{
						if (empty(trim($v)))
							$v = 'na';

						$cnt = 1;
						do
						{
							$lnk = $v.($cnt > 1 ? '-'.$cnt : '');

							$exists = $db->getRow(sprintf(
									"SELECT id_product FROM "._DB_PREFIX_."product_lang WHERE id_lang = %d AND id_product <> %d AND link_rewrite = '%s'",
									$k, 
									empty($product->id) ? 0 : $product->id,
									$db->escape($lnk)));
							$cnt++;
						} while ($exists && $cnt < 10000);

						if ($lnk != $v)
							$lnk_langs[$k] = $lnk;
					}

					$this->updProperty($log, $product, "link_rewrite", $lnk_langs);
				}

				// Sauvegarde du produit ici s'il y a des changements !
				$log->Commit();

				// Après la création du produit
				$log = $this->NewLogCtx($product, "Modification du produit %s", [ $product->reference ] );

				if (isset($data->date_add) && !empty($data->date_add) && 
					$data->date_add > '2002-01-01' && 
					$product->date_add != (string) $data->date_add)
				{
					$this->updProperty($log, $product, "date_add", $data->date_add);
				}

				if (version_compare(_PS_VERSION_, 1.7) >= 0 && isset($data->location))
				{
					$location = (string) $data->location;
					if ($location != $product->location)
					{
						$this->updProperty($log, $product, "location", $location);
						StockAvailable::setLocation($product->id, $location);
					}
				}

				// Sauvegarde du produit ici s'il y a des changements !
				$log->Commit();


				// On vérifie la référence fournisseur
				$this->associateProductToPolarisRef($polarisRef, $product->id);
				
				$no_group = intval($this->GetOptional($data, "", "group"));
				if ($no_group > 0)
				{
					// On vérifie que le modèle appartient au groupe					
					$no_group_actuel = NULL;					
					
					foreach ($this->SafeSQL("SELECT id_group FROM "._DB_PREFIX_."polarislink_products_group WHERE id_product = ".$product->id) as $row)
						$no_group_actuel = intval($row["id_group"]);
					
					if ($no_group_actuel != $no_group)
					{
						if ($no_group_actuel == 0)
							if ($db->query("INSERT INTO "._DB_PREFIX_."polarislink_products_group (id_product, id_group) VALUES (".$product->id.", ".$no_group.")") === false)
								throw new PrestaShopDatabaseException($db->getMsgError());
						else
							if ($db->query("UPDATE "._DB_PREFIX_."polarislink_products_group SET id_group = ".$no_group." WHERE id_product = ".$product->id) === false)
								throw new PrestaShopDatabaseException($db->getMsgError());
					}
				}
				
				// Association avec	les catégories de Polaris ...
				// Construction de l'association
				$root_category_id = (string) Configuration::get('PS_ROOT_CATEGORY');

				$ps_categories = $this->CategoryImporter->GetCategoriesById();
				
				$handled_categories = array();
				foreach ($data->category as $category)
				{
					$category_id = (string) $this->CategoryImporter->ImportData($category);
					if (!empty($category_id))
					{
						// On positionne la catégorie et toute la ligne jusqu'à la racine de la catégorie Polaris gérée
						$max = 100;
						do
						{
							$handled_categories[$category_id] = $category_id;
							
							$parent_category_id = isset($ps_categories[$category_id]) ? $ps_categories[$category_id]["id_parent"] : null;
							if (in_array($parent_category_id, $this->CategoryImporter->HandledCategoriesRoot))
								$parent_category_id = null;

							$category_id = $parent_category_id;
						} while ($max--> 0 && !empty($category_id) && $category_id != $root_category_id);			
					}
				}
				
				// Selon le mode de gestion des mots-clé, les tags sont des catégories !!
				if ($this->Link->MethodTags == 1)
				{					
					foreach ($data->tag as $tag)
					{
						$category_id = $this->CategoryImporter->ImportData($tag, "grouptag", "tag", "grtag");
						if (!empty($category_id))
							$handled_categories[$category_id] = $category_id;
					}				
				}				

				// On se calcule un tableau associatif avec pour chaque category_id existant, 0 si on ne fait rien, 1 si on ajoute, 2 si on retire la catégorie
				$categories_todo = [];				

				// Chargement du mapping
				$mapping = $this->CategoryImporter->getMappingInfoForProduct($product);
				
				// Calcul des catégories qu'on enlève
				foreach ($this->CategoryImporter->getHandledCategoriesForProduct($product) as $category_id)
				{
					if (!isset($handled_categories[$category_id]))
					{
						$categories_todo[$category_id] = 2;
						if (isset($mapping[$category_id]))
						{
							foreach ($mapping[$category_id] as $category_assoc)
								$categories_todo[$category_assoc] = 2;
						}
					}
				}

				// Calcul des catégories qu'on ajoute, on mappe au passage
				foreach ($handled_categories as $category_id)
				{
					$categories_todo[$category_id] = 1;							
					
					if (isset($mapping[$category_id]))
					{
						foreach ($mapping[$category_id] as $category_assoc)
							$categories_todo[$category_assoc] = 1;
					}
				}

				// Action : on efface, on ajoute réellement par rapport à ce qui existe ...
				$current_categories = $this->CategoryImporter->getCurrentProductCategories($product);
				$add = [];
				foreach ($categories_todo as $category_id => $action)
				{
					if ($action == 2)
					{
						if (isset($current_categories[$category_id]) && $current_categories[$category_id])
						{
							$product->deleteCategory($category_id, false);
							// Ajoute la catégorie à la reconstruction
							$this->Job->rebuild_categories[$category_id] = $category_id;
							unset($current_categories[$category_id]);

							$this->Log($product, "retrait lien avec catégorie #%s", [ $category_id ]);
						}
					}
					else if ($action == 1)
					{
						if (!isset($current_categories[$category_id]) || !$current_categories[$category_id])
						{
							$add[] = $category_id;
							$current_categories[$category_id] = $category_id;
						}
					}
				}

				if (count($add) > 0)
				{
					$product->addToCategories($add);
					foreach ($add as $category_id)
						$this->Log($product, "ajout lien avec catégorie #%s", [ $category_id ]);

					// On recharge les catégories
					$current_categories = $this->CategoryImporter->getCurrentProductCategories($product);
				}				

				$product_default_category = $this->CategoryImporter->computeDefaultCategoryForProduct($product);
				if (!empty($product_default_category) && $product_default_category != $product->id_category_default)
				{
					// Si la catégorie par défaut ne fait pas encore partie des catégories du produit, alors on 
					// corrige le tir
					if (!isset($current_categories[$product_default_category]))
						$this->Log($product, "ajout lien avec catégorie #%s", [ $product_default_category ]);

					$this->oneTimeUpdProperty("Catégorie par défaut", $product, "id_category_default", $product_default_category);
					$product->save();
				}
	
				// Tags ?
				if (!empty($data->tag))
					$this->Tags->addToProduct($log, $product, $data->tag);
				
				// On s'occupe des photos				
				if ($this->Link->OPT_ALLOW_I_PHOTOS)
				{					
					$currentPhotos = array();
					$currentPhotosByPoId = array();
					
					foreach ($product->getImages($this->Link->LangID) as $img)
					{
						$currentPhoto = array(
												"psId" => $img["id_image"],
												"poId" => NULL,
												"order" => 0,
												"c_order" => count($currentPhotos) + 1,
												"cover" => isset($img["cover"]) && !empty($img["cover"]),
												"used" => false
											);
																		
						$imgPath = _PS_PROD_IMG_DIR_.Image::getImgFolderStatic($img["id_image"]).$img["id_image"].".jpg";

						$currentPhoto["poId"] = $this->GetPolarisPhotoFor($imgPath);
						if ($currentPhoto["poId"])
							$currentPhotosByPoId[$currentPhoto["poId"]] = $currentPhoto["psId"];
						
						$currentPhotos[$currentPhoto["psId"]] = $currentPhoto;
					}
					
					// Création des photos produits manquantes									
					$order = 1;
					foreach ($photosList as $poId)
					{						
						$poId = trim($poId);
						if (!isset($currentPhotosByPoId[$poId]))
						{							
							// La photo Polaris existe, il ne reste qu'à faire l'association !!
							// Pour ça, on ajoute la photo au modèle
							$img = $this->BuildImage($product, $poId);
							if (!empty($img))
							{
								$currentPhotos[$img->id] = array(
																	"psId" => $img->id,
																	"poId" => $poId,
																	"order" => $order++,
																	"c_order" => 0,
																	"cover" => false,
																	"used" => true
																);
								$currentPhotosByPoId[$poId] = $img->id;
							}
						}
						else
						{
							$currentPhotos[$currentPhotosByPoId[$poId]]["used"] = true;
							if ($currentPhotos[$currentPhotosByPoId[$poId]]["order"] == 0)
								$currentPhotos[$currentPhotosByPoId[$poId]]["order"] = $order++;
						}
					}
					
					// Changement des positions des photos
					$firstPhoto = null;

					foreach ($currentPhotos as $photo)
					{
						// Photos non attribuée par Polaris => reléguée à la fin !
						if ($photo["order"] == 0)
							$photo["order"] = $order++;

						if ($photo["order"] != $photo["c_order"])
							$this->oneTimeUpdProperty(sprintf("Modif. ordre photo pour produit #%s", $product->id), new Image($photo["psId"]), "position", $photo["order"]);						

						if ($photo["order"] == 1)
							$firstPhoto = $photo;
					}

					// Changement de la photo de couverture					
					if (!empty($firstPhoto) && $firstPhoto["cover"] == false)
					{
						// Ménage
						Image::deleteCover($product->id);

						$img = new Image($firstPhoto["psId"]);
						if ($img)
						{
							$img->cover = 1;
							$img->update();
						}
					}
				}
				
				// Chargement des déclinaisons actuelles				
				if ($this->Link->OPT_SIZE_GRID)
				{					
					$id_group_attribute_sizegrid = $this->Link->AttrGroupForAttr0;
					$grille_taille = (string) $this->GetOptional($data, "sizegrid");
					if (!empty($grille_taille))
					{
						$grille_taille = mb_strtolower($grille_taille);						
						if (isset($this->SizeGridAttributesGroup[$grille_taille]))
							$id_group_attribute_sizegrid = $this->SizeGridAttributesGroup[$grille_taille];
					}
				}
				
				$currentCombinations = array();
				$currentCombinationsBySupplierRef = array();
				$lvlKeyToCombinationId = array();				
				
				$sql = "SELECT id_product_attribute, product_supplier_reference
						FROM "._DB_PREFIX_."product_supplier 
					WHERE id_product_attribute != 0 AND id_product=".(int)$product->id." AND id_supplier IN (".join(',', array($this->Link->SupplierID, $this->Link->SupplierBackupID)).")";
				$assocs = $this->SafeSQL($sql);
				/*if ($assocs === false)
					throw new Exception("Erreur au retrait des références fournisseurs");*/
				foreach ($assocs as $row)
					$currentCombinationsBySupplierRef[$row["product_supplier_reference"]] = (int) $row["id_product_attribute"];

				$sql = "SELECT pa.id_product_attribute AS id, pa.supplier_reference, 
								GROUP_CONCAT(CONCAT_WS(':', a.id_attribute_group, a.id_attribute) ORDER BY a.id_attribute_group ASC SEPARATOR ',') AS keylvl
						FROM "._DB_PREFIX_."product_attribute pa
							JOIN "._DB_PREFIX_."product_attribute_combination pac ON (pac.id_product_attribute = pa.id_product_attribute)
							JOIN "._DB_PREFIX_."attribute a ON (pac.id_attribute = a.id_attribute)
						WHERE pa.id_product=".(int)$product->id." GROUP BY pa.id_product_attribute";				
				$assocs = $this->SafeSQL($sql);
				/*if (!$assocs === false)
					throw new Exception("Erreur au retrait des déclinaisons existantes");*/

				foreach ($assocs as $row)
				{
					$currentCombinations[(int) $row["id"]] = false;
					$lvlKeyToCombinationId[$row["keylvl"]] = (int) $row["id"];

					if (!isset($currentCombinationsBySupplierRef[$row["supplier_reference"]]))
						$currentCombinationsBySupplierRef[$row["supplier_reference"]] = (int) $row["id_product_attribute"];
				}

				// Caractéristique ?
				if (count($this->OrderedFeaturesGroup) > 0)
				{
					$currentFeatures = [];
					foreach ($product->getFeatures() as $f)
						$currentFeatures[$f["id_feature"]] = $f["id_feature_value"];

					foreach ($this->OrderedFeaturesGroup as $k => $v)
					{
						$value = null;
						if ($v[0] == '*')
						{
							// La valeur est au niveau des articles ...						
							$v = substr($v, 1);
							foreach ($data->articles->article as $article)
							{								
								$value = $article->$v;
								if (!empty($value))
									break;								
							}
						}
						else if ($v[0] == '!')
						{							
							// Catégorie
							$v = substr($v, 1);
							foreach ($data->category as $c)
							{								
								if (isset($c["type"]) && $c["type"] == $v)
								{
									$value = $c->name;
									break;
								}
							}							
						}
						else
						{
							// Valeur au niveau du modèle							
							$value = $data->$v;
						}

						if ($value)
						{
							$feature_id = $this->getOrCreateFeature($k, $this->getLgValue($value, "checkAttributeName"));

							if (!isset($currentFeatures[$k]) || $feature_id != $currentFeatures[$k])
							{
								Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'feature_product` WHERE `id_product` = '.(int)$product->id.' AND `id_feature` = '.(int)$k);
								$product->addFeaturesToDB($k, $feature_id);		

								$this->Log($product, "chg caractéristique #%s, #%s -> #%s", 
									[ 	$k, 
										(isset($currentFeatures[$k]) && $currentFeatures[$k] != null) ? $currentFeatures[$k] : 'rien',
										$feature_id
									]);
							}
						}
						else if (isset($currentFeatures[$k]) && $currentFeatures[$k] != null)
						{
							// Il y avait une valeur, elle doit être supprimée !
							Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'feature_product` WHERE `id_product` = '.(int)$product->id.' AND `id_feature` = '.(int)$k);

							$this->Log($product, "chg caractéristique #%s, #%s -> #%s", 
									[ 	$k, 
										$currentFeatures[$k], 
										'rien'
									]);
						}
					}
				}
				
				// On construit les déclinaisons ici				
				if (!$is_virtual)
				{
					$attrsDone = array();
					foreach ($data->articles->article as $article)
					{
						$sku = $this->GetMandatory($article, "", "sku");
						if (!empty($sku)) $sku = trim($sku);
						if (!empty($sku) && empty((string) $article["deleted"]))
						{
							$cPolarisRef = $this->GetFirstReferencesFromList($sku);
							
							// Construction des liens attributs
							$lvlKey = "";
							$attrs = array();
							foreach ($this->OrderedAttributesGroup as $k => $v)
							{
								$value = $article->$v;
								if (!empty($value))
								{
									$a_value = $this->getLgValue($value, "checkAttributeName");
									$id_group_attribute = $k;

									if ($v == "size")
									{
										if ($this->Link->OPT_SIZE_GRID)
											$id_group_attribute = $id_group_attribute_sizegrid;

										// Si c'est une taille, on regarde si on doit pas concaténer avec
										// la valeur d'un autre niveau ...
										for ($i = 1; $i < 4; $i++)
										{
											$propName = "UseLevel".$i."ForSize";
											if ($this->Link->$propName > 0)
											{
												// C'est le cas pour ce niveau, on cherche
												// sa valeur
												$lvlName = "lvl".$i;
												$comp = $this->getLgValue($article->$lvlName, "checkAttributeName");
												if (!empty($comp))
												{
													$d = "";
													foreach ($comp as $k => $v)  {  if (!empty($v) ) { $d = $v;  break;  }  } // valeur par défaut

													if (!empty(trim($d)))
														foreach (array_keys($a_value) as $k)
															if (isset($comp[$k]))
																$a_value[$k] .= "/".trim($comp[$k]);
															else
																$a_value[$k] .= "/".trim($d);
												}
											}
										}										
									}
									
									$attr = $this->getOrCreateAttribute($id_group_attribute, $a_value, (string) $value["color"], (string) $value["position"]);
									$lvlKey .= ",".$id_group_attribute.":".$attr;
									$attrs[] = $attr;									
								}						
							}

							$attrKey = json_encode($attrs);
							if (isset($attrsDone[$attrKey]))
								continue;
							else
								$attrsDone[$attrKey] = $cPolarisRef;

							if (strlen($lvlKey) > 1)
							{
								$lvlKey = substr($lvlKey, 1);
								$isNew = false;
								$reAssociateAttribute = false;
								
								// Si la déclinaison existe dans l'état, on la charge et la marque comme étant utilisée
								$newCombination = null;

								// Recherche par attributs ...
								if (isset($lvlKeyToCombinationId[$lvlKey]))
								{
									$id = $lvlKeyToCombinationId[$lvlKey];
									$newCombination = new Combination($id);
									$currentCombinations[$id] = true;
								}
								else 
								{
									// Pas trouvé ? Recherche par équivalence de référence fournisseur ...
									foreach ($this->GetReferencesFromList($this->GetMandatory($article, "", "sku")) as $supplierRef)
									{
										if (isset($currentCombinationsBySupplierRef[$supplierRef]))	
										{
											$id = $currentCombinationsBySupplierRef[$supplierRef];										
											if (!$currentCombinations[$id])	// que si la combinaison n'a pas été utilisée pour une autre association
											{											
												$newCombination = new Combination($id);
												$currentCombinations[$id] = true;
												$reAssociateAttribute = true;
												// trouvé ?
												break;
											}
										}
									}								
								}

								if (!$newCombination)
								{								
									// Pas trouvé ?								
									// Création de la déclinaison								
									$newCombination = new Combination();
									$newCombination->id_product = $product->id;	
									$isNew = true;
								}
								
								$log = $this->NewLogCtx($newCombination, $isNew ? "Création déclinaison" : "Modification déclinaison");

								$this->updProperty($log, $newCombination, "reference", $cPolarisRef);
								$this->updProperty($log, $newCombination, "supplier_reference", $cPolarisRef);

								$c_ean13 = (string) $this->GetOptional($article, "", "ean");
								if (!empty($c_ean13))
									$this->updProperty($log, $newCombination, "ean13", checkEAN13($this->GetFirstReferencesFromList($c_ean13)));
								
								$this->updProperty($log, $newCombination, "price", checkPrice((!empty((string) $article->sp)) ? (float) $article->sp - $minPV : 0));
								// le pa est un remplaçant et non un incrément !
								//$this->updProperty($log, $newCombination, "wholesale_price", checkPrice((!empty((string) $article->wp)) ? (float) $article->wp - $minPA : 0));
								$this->updProperty($log, $newCombination, "wholesale_price", checkPrice((!empty((string) $article->wp)) ? (float) $article->wp : 0));

								$log->commit();

								// Après la sauvegarde
								$log = $this->NewLogCtx($newCombination, "Modification déclinaison");
								if ( version_compare(_PS_VERSION_, 1.7) >= 0 && isset($article->location) && !empty((string) $article->location))
								{
									$location = (string) $article->location;
									if ($newCombination->location != $location)
									{
										$this->updProperty($log, $newCombination, "location", $location);
										StockAvailable::setLocation($newCombination->id_product, $location, null, $newCombination->id);
									}
								}
								$log->commit();

								if ($isNew)
									$newCombination->setAttributes($attrs);
								else if ($reAssociateAttribute)
									$this->reassociateAttributes($newCombination, $attrs);
								
								$this->associateProductToPolarisRef($cPolarisRef, $product->id, $newCombination->id);
								
								// Photos !
								if (!empty((string) $article->photos) && $this->Link->OPT_ALLOW_I_PHOTOS)
								{
									$havePhotos = array();
									foreach ($newCombination->getWsImages() as $img)
										$havePhotos[] = $img["id"];								
									sort($havePhotos, SORT_NUMERIC);
									
									$wantedPhoto = array();								
									foreach ($this->GetReferencesFromList((string) $article->photos) as $poId)
									{
										if (isset($currentPhotosByPoId[$poId]))
										{
											$psId = $currentPhotosByPoId[$poId];
											if (!in_array($psId, $wantedPhoto))
												$wantedPhoto[] = $psId;
										}			
									}
									
									sort($wantedPhoto, SORT_NUMERIC);
									
									if (join(',', $havePhotos) != join(',', $wantedPhoto))
										$newCombination->setImages($wantedPhoto);
								}
							}
						}										
					}

					// Enfin, on efface les déclinaisons qui n'existent plus !!
					foreach ($currentCombinations as $id => $exists)
					{
						if (!$exists)
						{
							$c = new Combination($id);
							$product->deleteAttributeCombination($id);
							$this->Log($c, "Suppression déclinaison : reference %s", [$c->reference]);

							// Entretien du cache
							if ($this->RefsCache != null && !empty($c->reference))
								$this->RefsCache->setRef($c->reference, null, null);
						}
					}
				}				
				
				if ($this->Link->OPT_ALLOW_I_PHOTOS)
				{
					// Suppression des photos en trop ...
					$firstPhoto = null;
					foreach ($currentPhotos as $photo)
					{
						if (!$photo["used"])
						{
							$img = new Image($photo["psId"]);
							if ($img != NULL)
							{
								$img->delete();

								// Nettoyage lnk
								$lnk = _PS_PROD_IMG_DIR_.Image::getImgFolderStatic($img->id).$img->id.".jpg.polarislink";
								if (file_exists($lnk))
									@unlink($lnk);

								// nettoyage webp
								$img->image_format = "webp";
								$img->deleteImage();
							}
						}
						else if (empty($firstPhoto))
							$firstPhoto = $photo["psId"];
							
					}
					
					if ($firstPhoto && empty($product->getCoverWs()))
						$product->setCoverWs($firstPhoto);
				}
				
				// Déclinaison par défaut ?					
				$autoDetectDefault = (int)Product::getDefaultAttribute($product->id, 0, true);
				if (!empty($product->cache_default_attribute))
				{
					// Vérification ...
					$combination = new Combination($product->cache_default_attribute);
					if (!$combination->default_on || StockAvailable::getQuantityAvailableByProduct($combination->id_product, $combination->id) <= 0)
						$product->cache_default_attribute = NULL;
				}
				
				if (empty($product->cache_default_attribute) || empty($autoDetectDefault))
				{					
					$product->deleteDefaultAttributes();
					
					$attrs = Product::getProductAttributesIds($product->id);
					if (count($attrs) > 0)
					{
						$default_cmb_id = null;
						foreach ($attrs as $attr)
						{
							if (StockAvailable::getQuantityAvailableByProduct($product->id, $attr["id_product_attribute"]) > 0)
							{
								$default_cmb_id = $attr["id_product_attribute"];
								break;
							}
						}

						if (empty($default_cmb_id))
							$default_cmb_id = $attrs[0]["id_product_attribute"];

						$product->setDefaultAttribute($default_cmb_id);
						$this->Log($product, "Mise à jour de la déclinaison par défaut du produit => %s", [$default_cmb_id]);						
					}					
				}
				
				// On active le produit !
				if ($product->active == false && $this->Link->OPT_ACTIVE_PUBLISHED )
					$this->oneTimeUpdProperty("Activation auto. du produit", $product, "active", true);
			}
			else
			{
				// On va désactiver le produit, mais en prévoyant une redirection 301 automatique vers
				// sa catégorie parente
				if ($product != NULL && $product->active != 0)
				{
					$log = $this->NewLogCtx($product, "Dépublication produit %s", [ $product->reference ] );

					if ($product->redirect_type == "404" || $product->redirect_type == "")
						$this->updProperty($log, $product, "redirect_type", "301-category");

					// Infos avancées du produit
					if ($this->Link->OPT_UNPUBLISH_COOLDOWN)
					{
						$extraInfos = ProductExtraInfo::GetForProduct($product->id);

						// Si la visibilité est "Recherche" ou "Catalogue et recherche" :
						// - on vide les stocks 
						// - on met sa visibilité à aucune
						// - on met sa redirection à 301-category si c'est pas déjà le cas
						if ($product->visibility != "none" || !$extraInfos->date_depublication)
						{
							if ($product->visibility != "none")
								$this->updProperty($log, $product, "visibility", "none");
							
							// On vide les stocks de toutes les combinaisons
							$combinations = $product->getAttributeCombinations($this->Link->LangID);
							foreach ($combinations as $combination)
							{
								$stock = StockAvailable::getQuantityAvailableByProduct($product->id, $combination["id_product_attribute"]);
								if ($stock > 0)
									StockAvailable::setQuantity($product->id, $combination["id_product_attribute"], 0, null, false);
							}

							if (!$extraInfos->date_depublication)
							{
								// On met la date de dépublication du produit à aujourd'hui + 48h.
								$extraInfos->date_depublication = date("Y-m-d H:i:s", strtotime("+48 hours"));
								$extraInfos->save();
							}							
						}
					}
					else
					{						
						$this->updProperty($log, $product, "active", false);						
					}

					$log->Commit();
				}
			}
		}
		
		/**
		 * Scanne tous les produits et désactive automatiquement les produits :
		 * - qui doivent être désactivés automatiquement
		 */
		public function autoDisableProducts()
		{
			if (!$this->Link->OPT_UNPUBLISH_COOLDOWN)
				return;

			// On va plutôt faire ça par requête
			$sql = Db::getInstance();
			$results = $sql->executeS("SELECT id_product FROM "._DB_PREFIX_."polarislink_products WHERE date_depublication IS NOT NULL AND date_depublication < NOW()", true, false);
			if ($results === false)
				throw new Exception($sql->getMsgError());

			foreach ($results as $row)
			{
				$product = new Product($row["id_product"]);
				if ($product->id > 0)	// Produit trouvé !
				{
					$log = $this->NewLogCtx($product, "Désactivation automatique produit %s dépublié", [ $product->reference ] );
					$this->updProperty($log, $product, "active", false);
					$log->Commit();
				}

				// Purge du produit désactivé
				$sql->execute("UPDATE "._DB_PREFIX_."polarislink_products SET date_depublication = NULL WHERE id_product = ".(int)$product->id);
			}
		}
	}
	