<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	if (!defined('_PS_VERSION_'))
		exit;	
		
	require_once("PolarisLinkException.php");
	require_once("PolarisLinkSite.php");
	
	class PolarislinkRefCacheItem
	{
		public $id_product;
		public $id_product_attribute;
		public $backup;

		// Nouveau
		public function __construct($id_product, $id_product_attribute, $backup)
		{
			$this->id_product = (int) $id_product;
			$this->id_product_attribute = (int) $id_product_attribute;
			$this->backup = (bool) $backup;
		}

		public function toArray($ref = null)
		{
			return [ "id_product" => $this->id_product, "id_product_attribute" => $this->id_product_attribute, "ref" => $ref ];
		}			
	}

	/*
	 * Classe de gestion du cache de références
	 */
	class PolarisLinkRefCache
	{
		private $_RefMap = NULL;
		private $_EanMap = NULL;

		private $_LinkCfg = NULL;
		private $_Hit = 0;
		private $_Miss = 0;
		
		public function __construct(PolarisLinkSite $link)
		{
			$this->_LinkCfg = $link;			
		}
		
		/*
			Charge les références
		*/
		public function load()
		{
			$refMap = [];
			$eanMap = [];

			$db = Db::getInstance();
			
			$queries = [];
			
			// Les product_attributes
			$queries[] = "SELECT s.product_supplier_reference, s.id_product, s.id_product_attribute, (s.id_supplier = ".$this->_LinkCfg->SupplierBackupID.") as backup
					FROM " . _DB_PREFIX_ . "product_supplier AS s
					LEFT JOIN " . _DB_PREFIX_ . "product AS p ON (p.id_product = s.id_product)
					LEFT JOIN " . _DB_PREFIX_ . "product_attribute AS pa ON (pa.id_product = s.id_product AND pa.id_product_attribute = s.id_product_attribute)
					WHERE (s.id_supplier = ".$this->_LinkCfg->SupplierID." OR s.id_supplier = ".$this->_LinkCfg->SupplierBackupID.") AND p.id_product IS NOT NULL AND pa.id_product_attribute IS NOT NULL
					ORDER BY s.id_product, s.id_product_attribute";

			// Les références de produits seuls
			$queries[] = "SELECT s.product_supplier_reference, s.id_product, 0 as id_product_attribute, (s.id_supplier = ".$this->_LinkCfg->SupplierBackupID.") as backup
					FROM " . _DB_PREFIX_ . "product_supplier AS s
					LEFT JOIN " . _DB_PREFIX_ . "product AS p ON (p.id_product = s.id_product)
					WHERE s.id_product_attribute = 0 AND (s.id_supplier = ".$this->_LinkCfg->SupplierID." OR s.id_supplier = ".$this->_LinkCfg->SupplierBackupID.") 
					AND p.id_product IS NOT NULL
					AND NOT EXISTS (SELECT id_product FROM " . _DB_PREFIX_ . "product_attribute AS pa WHERE pa.id_product = s.id_product)
					ORDER BY s.id_product";

			foreach ($queries as $query)
			{
				$results = $db->executeS($query, true, false);
				if ($results === false)
					throw new PrestaShopDatabaseException($db->getMsgError());

				foreach ($results as $row)
				{
					$ean = (string) $row["product_supplier_reference"];

					// Alimentation du cache
					if (!isset($refMap[$ean]))
						$refMap[$ean] = new PolarislinkRefCacheItem($row["id_product"], $row["id_product_attribute"], $row["backup"]);
					else if ($refMap[$ean]->backup && !$row["backup"])	// S'il existe et qu'il est en backup alors qu'on a une entrée sans backup, alors il est maintenant en principal
						$refMap[$ean]->backup = false;
				}
			}

			// Les product_attributes
			$queries = [];
			$queries[] = "SELECT pa.ean13, pa.id_product, pa.id_product_attribute
							FROM " . _DB_PREFIX_ . "product_attribute AS pa LEFT JOIN " . _DB_PREFIX_ . "product AS p ON (p.id_product = pa.id_product)
							WHERE p.id_product IS NOT NULL AND pa.ean13 != ''
							ORDER BY pa.id_product, pa.id_product_attribute";

			// Les références de produits seuls
			$queries[] = "SELECT p.ean13, p.id_product, 0 AS id_product_attribute
							FROM ". _DB_PREFIX_ . "product AS p
							WHERE 
							p.ean13 != '' AND p.id_product IS NOT NULL 
							AND NOT EXISTS (SELECT id_product FROM " . _DB_PREFIX_ . "product_attribute AS pa WHERE pa.id_product = p.id_product)
							ORDER BY p.id_product";

			foreach ($queries as $query)
			{
				$results = $db->executeS($query, true, false);
				if ($results === false)
					throw new PrestaShopDatabaseException($db->getMsgError());

				foreach ($results as $row)
				{
					$ean = (string) $row["ean13"];

					// Alimentation du cache
					if (!isset($eanMap[$ean]))
						$eanMap[$ean] = new PolarislinkRefCacheItem($row["id_product"], $row["id_product_attribute"], false);					
				}
			}

			$this->_RefMap = $refMap;
			$this->_EanMap = $eanMap;
		}

		/*
			Enregistre une nouvelle référence
		*/
		public function setRef($reference, $id_product, $id_product_attribute)
		{
			if (!empty($reference) && $this->_RefMap != null)
			{
				if (isset($this->_RefMap[$reference]))
				{
					// On a déjà une référence
					if (empty($id_product))
					{
						// Qu'il faut supprimer
						// echo "del ref for EAN ".$reference."\n";
						unset($this->_RefMap[$reference]);
					}
					else
					{
						$current = $this->_RefMap[$reference];
						$id_product = (int) $id_product;
						$id_product_attribute = (int) $id_product_attribute;

						if ($current->id_product != $id_product || $current->id_product_attribute != $id_product_attribute)
						{
							// Qu'il faut remplacer
							// echo "new ref for EAN ".$reference." : from ".$current->id_product."/".$current->id_product_attribute." to ".$id_product."/".$id_product_attribute."\n";
							$this->_RefMap[$reference] = new PolarislinkRefCacheItem($id_product, $id_product_attribute, false);				
						}
					}
				}
				else
				{
					// La référence n'existe pas ...
					if (!empty($id_product))
					{
						// Et on doit la créer
						// echo "add ref for EAN ".$reference." : to ".$id_product."/".$id_product_attribute."\n";
						$this->_RefMap[$reference] = new PolarislinkRefCacheItem($id_product, $id_product_attribute, false);				
					}
				}
			}
		}

		/*
			Recherche un produit et sa combinaison prestashop correspondant à la référence Polaris, ou au code barre gencode, ou 
			false si non trouvé dans le cache
		*/
		public function get(
			$references, 
			$eans = NULL, 
			$searchInPolarisBackup = false)
		{
			$this->_Hit++;

			if ($this->_RefMap == null)
				$this->load();

			if ($references != null && count($references) > 0)
			{
				foreach ($references as $ref)
				{
					if (isset($this->_RefMap[$ref]))
					{
						$candidate = $this->_RefMap[$ref];
						if ($searchInPolarisBackup || !$candidate->backup)
							return $candidate->toArray($ref);
					}
				}
			}

			if ($eans != null && count($eans) > 0)
			{
				foreach ($eans as $ean)
				{
					if (isset($this->_EanMap[$ean]))
						return $this->_EanMap[$ean]->toArray();
				}
			}
			
			$this->_Miss++;
			return NULL;
		}

		/*
			Recherche tous les produits prestashop correspondant à la référence Polaris
		*/
		public function getAll($references)
		{
			$this->_Hit++;

			if ($this->_RefMap == null)
				$this->load();

			$results = [];
			if ($references != null && count($references) > 0)
			{
				foreach ($references as $ref)
				{
					if (isset($this->_RefMap[$ref]))
					{
						$candidate = $this->_RefMap[$ref];
						$results[] = $candidate->toArray($ref);
					}
				}
			}

			$this->_Miss++;
			return $results;
		}
	} 
	