<?php
	/*
        Dispatch les déclinaisons des produits en plusieurs produits en ne gardant
        que certains attributs jumulés
		
		Usage :
		split_products <attributes>
                    
        Avec :
        - attributes : liste des numéros des attributs à garder ensembles, séparée par des virgules
	*/	
    
    $opts = getopt("", array("cfg:", "mirror:"));

	if (!isset($opts["mirror"]) || empty($opts["mirror"]) || !isset($opts["cfg"]) || empty($opts["cfg"]))
	{
        echo "Usage php ".$argv[0]." --cfg </path/to/config.inc.php directory> --mirror <mirroir1.tests.vega-net.net>\n";
        die();
    }
   
	// Autodétection de l'emplacement de Prestashop (attention au lien symbolique sur plateforme de dev)    
	    
	// Inclusion de la configuration
    require($opts["cfg"].'/config.inc.php');		
    
	class MirrorMaker
	{
        private $db;
        private $systemLang;

		public function __construct()
		{
            $this->systemLang = Configuration::get('PS_LANG_DEFAULT');
		}
		
		/*
			Enregistre une erreur sur le journal de Prestashop
			et également sur la console
		*/
		function logError($msg, $level = 3, $objClass = null, $objId = 0)
		{
			PrestaShopLogger::addLog("Mirror : ".$msg, $level, 1, "PolarisLink", $objId, true);
			echo $msg."\n";
        }
        
        /*
			Déplace une combinaison dans un autre produit
		*/
		function moveProductAttributeToAnotherProduct($combination, $product)
		{
			// Création nouvelle combinaison
            $newCombination = new Combination();
            
            foreach (Combination::$definition["fields"] as $field => $def)
                $newCombination->$field = $combination->$field;

			$newCombination->id_product = $product->id;
            $newCombination->save();
            
            // Copie des attributs
			$attrs = array();
			foreach ($combination->getAttributesName($this->systemLang) as $attr)
				$attrs[] = $attr["id_attribute"];									
            $newCombination->setAttributes($attrs);
            
            // Recopions les références fournisseurs
            foreach (ProductSupplier::getSupplierCollection($combination->id_product, false) as $ps)
            {
                if ($ps->id_product_attribute == $combination->id)
                {
                    $nps = new ProductSupplier();
                    foreach (ProductSupplier::$definition["fields"] as $field => $def)
                        $nps->$field = $ps->$field;

                    $nps->id_product = $newCombination->id_product;
                    $nps->id_product_attribute = $newCombination->id;

                    $nps->save();
                }
            }
            
            return $newCombination->id;
        }

        /*
         * Supprime du produit les photos qui ne sont pas utilisées par au moins une catégorie
         */
        function menagePhoto($product)
        {
            $id_product = $product->id;
            foreach ($this->db->ExecuteS("SELECT id_image FROM "._DB_PREFIX_."image WHERE id_product = ".$id_product." AND id_image NOT IN (SELECT id_image FROM "._DB_PREFIX_."product_attribute_image WHERE id_product_attribute IN (SELECT id_product_attribute FROM "._DB_PREFIX_."product_attribute WHERE id_product = ".$id_product.")) ORDER BY id_image") as $row)
            {
                $img = new Image($row["id_image"]);
                $img->delete();                
            }
            Image::deleteCover($id_product);

            foreach ($product->getImages($this->systemLang) as $row)
            {
                $product->setCoverWs($row["id_image"]);
                break;
            }
        }
        
		/*
			Importation
		*/
		function run($mirrorUrl)
		{
            // Base de données
            $this->db = Db::getInstance();
            if (!$this->db)
                die ("Erreur d'accès à la base de données");

            // Pas de cache !
            $this->db->disableCache();            
            
            $this->db->execute(sprintf("UPDATE "._DB_PREFIX_."configuration SET value = CONCAT('MIROIR ', value) WHERE name = 'PS_SHOP_NAME'"));
            $this->db->execute(sprintf("UPDATE "._DB_PREFIX_."configuration SET value = 'admin@vega-info.fr' WHERE name = 'PS_SHOP_EMAIL'"));
            $this->db->execute(sprintf("UPDATE "._DB_PREFIX_."configuration SET value = '%s' WHERE name IN ('PS_SHOP_DOMAIN', 'PS_SHOP_DOMAIN_SSL')", $mirrorUrl));
            $this->db->execute(sprintf("UPDATE "._DB_PREFIX_."shop_url SET domain = '%s', domain_ssl = '%s', physical_uri = '/'", $mirrorUrl, $mirrorUrl));

            // RGPD : anonymisation
            $this->db->execute("UPDATE "._DB_PREFIX_."customer SET lastname = SUBSTRING(MD5(lastname), 8), email = CONCAT('mail-', id_customer, '@vega-info.fr')");
            
            $e = new Employee();
            $e->id_profile = _PS_ADMIN_PROFILE_;
            $e->firstname = "VEGA";
            $e->lastname = "INFORMATIQUE";
            $e->email = "julienv@vega-info.fr";
            $e->setWsPasswd("VegaSav33");
            $e->save();
        }
    }
    
	$util = new MirrorMaker();
    $util->run($opts["mirror"]);
